﻿'Programed by NAS6


'オイラー法
'速度と加速度はローレンツ変換、/CNST_Cで値保持、座標は無変換
Public Class Euler
    Public CNST_G As Double = 0.00000000006673
    Public CNST_C As Double = 299792458.0

    Public mp() As MassPoint
    Public dt As Double
    Public dx(,) As Vector
    Public nrm(,) As Vector
    Public av(,) As Vector
    Public r(,) As Double

    Public rdx(,) As Double


    'DoubleCeil補助関数
    Public Function AndExponent(ByVal db() As Byte, ByVal ee() As Byte) As Byte()
        Dim ret(7) As Byte
        Dim i As Integer
        For i = 0 To 5
            ret(i) = db(i)
        Next
        ret(6) = (db(6) And (15))
        ret(6) = (ret(6)) Or (ee(0) << (4))
        ret(7) = db(7)
        Return ret
    End Function

    Public Function Exponent2Bytes(ByVal i As Int64) As Byte()
        Dim ret() As Byte = BitConverter.GetBytes(i)
        Return ret
    End Function

    Public Function Bytes2Exponent(ByVal b() As Byte) As Int64
        Dim ret(7) As Byte
        ret(0) = ((b(6) And (240)) >> (4))
        Return BitConverter.ToInt64(ret, 0)
    End Function

    Public Function AndFraction(ByVal db() As Byte, ByVal df() As Byte) As Byte()
        Dim ret(7) As Byte
        Dim i As Integer
        For i = 0 To 5
            ret(i) = df(i)
        Next
        ret(6) = (df(6) And (15))
        ret(6) = (ret(6)) Or (db(6) And (240))
        ret(7) = db(7)
        Return ret
    End Function

    Public Function Fraction2Bytes(ByVal i As Int64) As Byte()
        Dim ret() As Byte = BitConverter.GetBytes(i)
        Return ret
    End Function

    Public Function Bytes2Fraction(ByVal b() As Byte) As Int64
        Dim ret(7) As Byte
        Dim i As Integer
        For i = 0 To 5
            ret(i) = b(i)
        Next
        ret(6) = (b(6) And (15))
        ret(7) = 0
        Return BitConverter.ToInt64(ret, 0)
    End Function

    Public Function Bytes2Double(ByVal b() As Byte) As Double
        Return BitConverter.ToDouble(b, 0)
    End Function

    Public Function Double2Bytes(ByVal d As Double) As Byte()
        Dim ret() As Byte = BitConverter.GetBytes(d)
        Return ret
    End Function

    '1/2^(52-n)の桁上げ
    Public Function DoubleCeil(ByVal d As Double, ByVal n As Integer) As Double
        Dim db() As Byte = Double2Bytes(d)
        Dim di As Int64 = Bytes2Fraction(db)
        Dim o As Int64 = 1
        Dim p As Integer
        For p = 1 To n - 1
            o = o * 2
        Next
        di = di + o
        Dim df() As Byte = Fraction2Bytes(di)
        If (df(6) And (16)) Then
            Dim ei As Int64 = Bytes2Exponent(db)
            ei = ei + 1
            Dim ee() As Byte = Exponent2Bytes(ei)
            db = AndExponent(db, ee)
        End If
        db = AndFraction(db, df)
        Dim ret As Double = Bytes2Double(db)
        Return ret
    End Function

    '速度加速
    Public Function VelocityAccl2D(ByVal v As Vector, ByVal a As Vector, ByVal dt As Double) As Vector
        If a.Abs() = 0.0 Then
            Return v
        End If
        If 1.0 < v.Abs() Then
            If v.x(1) <= v.x(0) Then
                v.x(1) = Math.Sqrt(1.0 - v.x(0) * v.x(0))
            End If
            If v.x(0) < v.x(1) Then
                v.x(0) = Math.Sqrt(1.0 - v.x(1) * v.x(1))
            End If
        End If

        Dim va As Double = v.Abs()
        Dim aa As Double = a.Abs()

        Dim vab As Double = Math.Tanh(aa * dt)
        Dim vac As Double = ((va + vab) / (va * vab + 1.0))

        Dim ret As Vector = v + a * dt

        If 1.0 < vac Then
            ret = ret / vac
        End If

        Return ret
    End Function

    '速度加速
    Public Function VelocityAccl3D(ByVal v As Vector, ByVal a As Vector, ByVal dt As Double) As Vector
        If a.Abs() = 0.0 Then
            Return v
        End If
        If 1.0 < v.Abs() Then
            If v.x(1) <= v.x(0) And v.x(2) <= v.x(0) Then
                v.x(1) = Math.Sqrt((1.0 - v.x(0) * v.x(0)) / 2.0)
                v.x(2) = Math.Sqrt((1.0 - v.x(0) * v.x(0)) / 2.0)
            End If
            If v.x(2) <= v.x(1) And v.x(0) < v.x(1) Then
                v.x(2) = Math.Sqrt((1.0 - v.x(1) * v.x(1)) / 2.0)
                v.x(0) = Math.Sqrt((1.0 - v.x(1) * v.x(1)) / 2.0)
            End If
            If v.x(0) < v.x(2) And v.x(1) < v.x(2) Then
                v.x(0) = Math.Sqrt((1.0 - v.x(2) * v.x(2)) / 2.0)
                v.x(1) = Math.Sqrt((1.0 - v.x(2) * v.x(2)) / 2.0)
            End If
        End If

        Dim va As Double = v.Abs()
        Dim aa As Double = a.Abs()

        Dim vab As Double = Math.Tanh(aa * dt)
        Dim vac As Double = ((va + vab) / (va * vab + 1.0))

        Dim ret As Vector = v + a * dt

        If 1.0 < vac Then
            ret = ret / vac
        End If

        Return ret
    End Function

    '速度合成
    Public Function VelocityAdd2D(ByVal v0 As Vector, ByVal v1 As Vector) As Vector
        VelocityAdd2D = New Vector(v0.x.Length)
        Dim ra As Double = v1.Abs()
        ra = DoubleCeil(ra, 1)
        Dim i As Integer
        If v0.Abs() = 0.0 Then
            If 1.0 < ra Then
                For i = 0 To v0.x.Length - 1
                    VelocityAdd2D.x(i) = v1.x(i) / ra
                Next
            Else
                VelocityAdd2D = v1
            End If
            Return VelocityAdd2D
        End If
        ra = v0.Abs()
        ra = DoubleCeil(ra, 1)
        If v1.Abs() = 0.0 Then
            If 1.0 < ra Then
                For i = 0 To v0.x.Length - 1
                    VelocityAdd2D.x(i) = v0.x(i) / ra
                Next
            Else
                VelocityAdd2D = v0
            End If
            Return VelocityAdd2D
        End If


        Dim ret As Vector = New Vector(3)
        Dim v As Double = v0.Abs()
        Dim v11 As Vector = New Vector(3)
        Dim v00 As Vector = New Vector(3)
        For i = 0 To v0.x.Length - 1
            v11.x(i) = v1.x(i)
            v00.x(i) = v0.x(i)
        Next
        v11.x(i) = 0.0
        v00.x(i) = 0.0
        Dim ux As Vector = ret.UnitVec(0)
        Dim crs As Vector = v00.Cross(ux)

        Dim theta As Double = v00.Theta(ux)
        If Double.IsNaN(theta) Then
            theta = v11.Theta(ux)
            If Double.IsNaN(theta) Then
                theta = 0.0
            Else
                v00 = v11.RotAxis(crs, theta)
            End If
        Else
            v00 = v00.RotAxis(crs, theta)
        End If
        v11 = v11.RotAxis(crs, theta)
        If v11.x(0) * v = -1.0 Then
            v11.x(0) = -0.99999999999999989
        End If

        ret.x(0) = ((v11.x(0) + v) / (v11.x(0) * v + 1.0))
        ret.x(1) = ((v11.x(1)) / (v11.x(0) * v + 1.0)) * Math.Sqrt(1.0 - v * v)
        ret.x(2) = ((v11.x(2)) / (v11.x(0) * v + 1.0)) * Math.Sqrt(1.0 - v * v)
        ret = ret.RotAxis(crs, -theta)
        ra = ret.Abs()
        ra = DoubleCeil(ra, 1)
        If 1.0 < ra Then
            For i = 0 To v0.x.Length - 1
                VelocityAdd2D.x(i) = ret.x(i) / ra
            Next
        Else
            For i = 0 To v0.x.Length - 1
                VelocityAdd2D.x(i) = ret.x(i)
            Next
        End If
    End Function

    '速度合成
    Public Function VelocityAdd3D(ByVal v0 As Vector, ByVal v1 As Vector) As Vector
        VelocityAdd3D = New Vector(v0.x.Length)
        Dim ra As Double = v1.Abs()
        ra = DoubleCeil(ra, 1)
        Dim i As Integer
        If v0.Abs() = 0.0 Then
            If 1.0 < ra Then
                For i = 0 To v0.x.Length - 1
                    VelocityAdd3D.x(i) = v1.x(i) / ra
                Next
            Else
                VelocityAdd3D = v1
            End If
            Return VelocityAdd3D
        End If
        ra = v0.Abs()
        ra = DoubleCeil(ra, 1)
        If v1.Abs() = 0.0 Then
            If 1.0 < ra Then
                For i = 0 To v0.x.Length - 1
                    VelocityAdd3D.x(i) = v0.x(i) / ra
                Next
            Else
                VelocityAdd3D = v0
            End If
            Return VelocityAdd3D
        End If


        Dim ret As Vector = New Vector(3)
        Dim v As Double = v0.Abs()
        Dim v11 As Vector = New Vector(3)
        Dim v00 As Vector = New Vector(3)
        For i = 0 To v0.x.Length - 1
            v11.x(i) = v1.x(i)
            v00.x(i) = v0.x(i)
        Next
        Dim ux As Vector = ret.UnitVec(0)
        Dim crs As Vector = v00.Cross(ux)

        Dim theta As Double = v00.Theta(ux)
        If Double.IsNaN(theta) Then
            theta = v11.Theta(ux)
            If Double.IsNaN(theta) Then
                theta = 0.0
            Else
                v00 = v11.RotAxis(crs, theta)
            End If
        Else
            v00 = v00.RotAxis(crs, theta)
        End If
        v11 = v11.RotAxis(crs, theta)

        ret.x(0) = ((v11.x(0) + v) / (v11.x(0) * v + 1.0))
        ret.x(1) = ((v11.x(1)) / (v11.x(0) * v + 1.0)) * Math.Sqrt(1.0 - v * v)
        ret.x(2) = ((v11.x(2)) / (v11.x(0) * v + 1.0)) * Math.Sqrt(1.0 - v * v)
        ret = ret.RotAxis(crs, -theta)
        ra = ret.Abs()
        ra = DoubleCeil(ra, 1)
        If 1.0 < ra Then
            For i = 0 To v0.x.Length - 1
                VelocityAdd3D.x(i) = ret.x(i) / ra
            Next
        Else
            For i = 0 To v0.x.Length - 1
                VelocityAdd3D.x(i) = ret.x(i)
            Next
        End If
    End Function

    'シュワルツシルト半径
    Public Function GetSRadius(ByVal mass As Double, ByVal cc As Double, ByVal cg As Double) As Double
        Return ((2.0 * cg * mass) / (cc * cc))
    End Function

    'シュワルツシルト補正項
    Public Function ToSchwartz(ByVal v As Double) As Double
        ToSchwartz = 3.0 * v * v
    End Function

    '加速度を求める
    Public Function GetA(ByVal r As Double, ByVal m As Double, ByVal mr As Double, ByVal v As Double) As Double
        If r = 0.0 Then
            Return 0.0
        End If
        Dim a As Double = 1.0
        If mr <= r Then
            mr = r
        Else
            a = r / mr
        End If
        Dim g As Double = CNST_G * (m / mr / mr) * a
        g = g * (1.0 + ToSchwartz(v))
        If g < 0.0 Or Double.IsNaN(g) Then
            Return 0.0
        End If
        GetA = (g / CNST_C)
    End Function



    'オイラー法
    Public Sub UpdateFrame()
        Dim i As Integer
        Dim j As Integer
        Dim k As Integer

        '設定
        For i = 0 To mp.Length - 1
            For k = 0 To mp(0).x.x.Length - 1
                mp(i).x1.x(k) = mp(i).x.x(k)
                mp(i).x0.x(k) = mp(i).x.x(k)
                mp(i).v1.x(k) = mp(i).v.x(k)
                mp(i).a.x(k) = 0.0
            Next
        Next

        '加速度を求める
        For i = 0 To mp.Length - 1 - 1
            For j = i + 1 To mp.Length - 1
                dx(i, j) = mp(i).x1 - mp(j).x1
                nrm(i, j) = dx(i, j) / dx(i, j).Abs()
                r(i, j) = dx(i, j).Abs()

                Dim a1 As Double
                Dim a As Vector
                '軌道成分速度
                Dim ov As New Vector(3)
                Dim dx2 As New Vector(3)
                Dim dx3 As New Vector(3)
                Dim dx1 As New Vector(3)
                Dim vv1 As New Vector(3)
                For k = 0 To dx(i, j).x.Length - 1
                    dx1.x(k) = dx(i, j).x(k)
                    vv1.x(k) = mp(i).v1.x(k)
                Next
                If Not dx1.Abs = 0.0 Then
                    If dx1 = vv1 Then
                        If dx1 = dx2.UnitVec(0) Then
                            dx2 = dx1.Cross(dx2.UnitVec(1))
                        Else
                            dx2 = dx1.Cross(dx2.UnitVec(0))
                        End If
                    Else
                        dx2 = dx1.Cross(vv1)
                    End If
                    If dx1 = dx2 Then
                        'ERROR
                        dx3 = dx1
                    Else
                        dx3 = dx1.Cross(dx2)
                    End If
                    ov = vv1.ProjectAxis(dx3)
                End If

                a1 = GetA(r(i, j), mp(j).mass, mp(j).r, ov.Abs())

                a = nrm(i, j) * a1 * -1.0

                For k = 0 To a.x.Length - 1
                    av(i, j).x(k) = a.x(k)
                Next

                For k = 0 To dx(i, j).x.Length - 1
                    dx1.x(k) = dx(i, j).x(k)
                    vv1.x(k) = mp(j).v1.x(k)
                Next
                If Not dx1.Abs = 0.0 Then
                    If dx1 = vv1 Then
                        If dx1 = dx2.UnitVec(0) Then
                            dx2 = dx1.Cross(dx2.UnitVec(1))
                        Else
                            dx2 = dx1.Cross(dx2.UnitVec(0))
                        End If
                    Else
                        dx2 = dx1.Cross(vv1)
                    End If
                    If dx1 = dx2 Then
                        'ERROR
                        dx3 = dx1
                    Else
                        dx3 = dx1.Cross(dx2)
                    End If
                    ov = vv1.ProjectAxis(dx3)
                End If

                a1 = GetA(r(i, j), mp(i).mass, mp(i).r, ov.Abs())

                a = nrm(i, j) * a1

                For k = 0 To a.x.Length - 1
                    av(j, i).x(k) = a.x(k)
                Next

            Next
        Next



        'パラメータ適用
        For i = 0 To mp.Length - 1
            Dim vv As Vector = New Vector(mp(0).x.x.Length)
            For j = 0 To mp.Length - 1
                For k = 0 To mp(i).v.x.Length - 1
                    mp(i).a.x(k) = mp(i).a.x(k) + av(i, j).x(k)
                Next
            Next
            Dim av2 As Vector = New Vector(0)
            If mp(i).v.x.Length - 1 = 2 Then
                av2 = VelocityAccl3D(New Vector(3), mp(i).a, dt)
            ElseIf mp(i).v.x.Length - 1 = 1 Then
                av2 = VelocityAccl2D(New Vector(2), mp(i).a, dt)
            End If
            For k = 0 To mp(i).v.x.Length - 1
                vv.x(k) = av2.x(k)
            Next
            If mp(0).x.x.Length = 3 Then
                mp(i).v = VelocityAdd3D(mp(i).v, vv)
            ElseIf mp(0).x.x.Length = 2 Then
                mp(i).v = VelocityAdd2D(mp(i).v, vv)
            End If

            If 1.0 < mp(i).v.Abs() Then
                i = i
            End If

            For k = 0 To mp(i).x.x.Length - 1
                mp(i).x.x(k) = mp(i).x.x(k) + mp(i).v.x(k) * dt * CNST_C
            Next
            For j = i To mp.Length - 1
                rdx(i, j) = r(i, j)
                rdx(j, i) = r(j, i)
            Next
        Next

    End Sub

    'オイラー法初期設定
    Public Sub Init(ByVal pmp() As MassPoint, ByVal pdt As Double, ByVal cc As Double, ByVal cg As Double)
        Me.mp = pmp
        Me.dt = pdt

        If Not cc = 0.0 Then
            Me.CNST_C = cc
        End If
        If Not cg = 0.0 Then
            Me.CNST_G = cg
        End If

        ReDim rdx(mp.Length - 1, mp.Length - 1)
        ReDim dx(mp.Length - 1, mp.Length - 1)
        ReDim nrm(mp.Length - 1, mp.Length - 1)
        ReDim av(mp.Length - 1, mp.Length - 1)
        ReDim r(mp.Length - 1, mp.Length - 1)

        Dim i As Integer
        Dim j As Integer

        For i = 0 To mp.Length - 1
            For j = i To mp.Length - 1
                dx(i, j) = New Vector(mp(0).x.x.Length)
                nrm(i, j) = New Vector(mp(0).x.x.Length)
                av(i, j) = New Vector(mp(0).x.x.Length)
                r(i, j) = 0.0

                dx(j, i) = New Vector(mp(0).x.x.Length)
                nrm(j, i) = New Vector(mp(0).x.x.Length)
                av(j, i) = New Vector(mp(0).x.x.Length)
                r(j, i) = 0.0
            Next
        Next

        Dim fw As Double
        For i = 0 To mp.Length - 1
            For j = i To mp.Length - 1
                fw = 0.0
                If Not i = j Then
                    dx(i, j) = mp(i).x1 - mp(j).x1
                    dx(j, i) = mp(j).x1 - mp(i).x1
                Else
                    dx(i, j) = mp(i).x1 - mp(j).x0
                End If
                For k = 0 To mp(0).x.x.Length - 1
                    fw += dx(i, j).x(k) * dx(i, j).x(k)
                Next
                rdx(i, j) = Math.Sqrt(fw)
                rdx(j, i) = Math.Sqrt(fw)
            Next
        Next
    End Sub

    'オイラー法初期設定
    Public Sub Init(ByVal pmp() As MassPoint, ByVal pdt As Double)
        Init(pmp, pdt, 0.0, 0.0)
    End Sub

End Class
