﻿'Programed by NAS6

'無限対数軸
Public Class Infinity
    Implements IComparable
    Implements ICloneable

    Public Range As Double = 1.0  '対数軸の範囲
    Public Base As Double = 2.0   '基数


    Public Function Clone() As Object Implements ICloneable.Clone
        Dim ret As New Infinity(Me.Range, Me.Base)
        Return ret
    End Function

    Public Function CompareTo(obj As Object) As Integer Implements IComparable.CompareTo
        CompareTo = 0
        Dim lh As Infinity = Me
        Dim rh As Infinity = obj
        If Not (lh.Range = rh.Range) Then
            CompareTo = CompareTo Or 1
        End If
        If Not (lh.Base = rh.Base) Then
            CompareTo = CompareTo Or 2
        End If
    End Function

    Public Overrides Function Equals(obj As Object) As Boolean
        Dim ret As Integer = CompareTo(obj)
        If ret = 0 Then
            Return True
        Else
            Return False
        End If
    End Function

    Public Shared Operator =(ByVal lh As Infinity, ByVal rh As Infinity) As Boolean
        Dim ret As Integer = lh.CompareTo(rh)
        If ret = 0 Then
            Return True
        Else
            Return False
        End If
    End Operator

    Public Shared Operator <>(ByVal lh As Infinity, ByVal rh As Infinity) As Boolean
        Return (Not lh = rh)
    End Operator

    Public Sub New()
        Range = 1.0
        Base = 2.0
    End Sub

    Public Sub New(ByVal r As Double, ByVal b As Double)
        Range = r
        Base = b
    End Sub

    '無限対数軸から通常軸へ
    Public Function FromLogAxis(ByVal x As Double) As Double
        If Double.IsNaN(x) Or Range <= Math.Abs(x) Then
            FromLogAxis = Double.PositiveInfinity
        Else
            FromLogAxis = Range / (Math.Log(Math.Abs(x) / Range) / Math.Log(1.0 / Base))
            If Double.IsNaN(FromLogAxis) Or Double.IsInfinity(FromLogAxis) Then
                FromLogAxis = Double.PositiveInfinity
            End If
        End If
        If x < 0.0 Then
            FromLogAxis *= -1.0
        End If
    End Function

    '通常軸から無限対数軸へ
    Public Function ToLogAxis(ByVal x As Double) As Double
        Dim c As Double = Range / Math.Abs(x)
        If Double.IsNaN(c) Or Double.IsInfinity(c) Then
            ToLogAxis = 0.0
        Else
            ToLogAxis = Range * (Math.Pow(1.0 / Base, c))
            If Double.IsNaN(ToLogAxis) Or Double.IsInfinity(ToLogAxis) Then
                ToLogAxis = 0.0
            End If
        End If
        If x < 0.0 Then
            ToLogAxis *= -1.0
        End If
    End Function

    '無限遠透視射影
    Public Function FrustumInf0(ByVal v As Vector) As Vector
        Dim VecWK As New Vector(4)
        Dim VecWK2 As Vector = v.Clone()
        VecWK2 = VecWK2.Homogeneous()

        VecWK.x(0) = 1.0
        VecWK.x(1) = (Range - Me.ToLogAxis(VecWK2.x(3))) * VecWK2.x(1) / Range
        VecWK.x(2) = (Range - Me.ToLogAxis(VecWK2.x(3))) * VecWK2.x(2) / Range
        VecWK.x(3) = 0.0

        Return VecWK
    End Function

    '逆無限遠透視射影
    Public Function InvFrustumInf0(ByVal v As Vector, ByVal z As Double) As Vector
        Dim VecWK As New Vector(4)
        Dim VecWK2 As Vector = v.Clone()
        VecWK2 = VecWK2.Homogeneous()

        VecWK.x(0) = 1.0
        VecWK.x(1) = Range * VecWK2.x(1) / (Range - Me.ToLogAxis(z))
        VecWK.x(2) = Range * VecWK2.x(2) / (Range - Me.ToLogAxis(z))
        VecWK.x(3) = z / Range

        Return VecWK
    End Function

    '無限遠透視射影
    Public Function FrustumInf(ByVal v As Vector) As Vector
        Dim VecWK As Vector = FrustumInf0(v) / Range
        Return VecWK
    End Function

    '逆無限遠透視射影
    Public Function InvFrustumInf(ByVal v As Vector, ByVal z As Double) As Vector
        Dim VecWK As Vector = InvFrustumInf0(v, z) * Range
        Return VecWK
    End Function

End Class
