﻿'Programed by NAS6

'ルンゲ-クッタ法
'速度と加速度はローレンツ変換、/CNST_Cで値保持、座標は実座標
Public Class RngKt
    Public CNST_G As Double = 0.00000000006673
    Public CNST_C As Double = 299792458.0



    Public dms As Integer
    Public n As Integer
    Public mp() As MassPoint
    Public dt As Double


    Public rdx(,) As Double
    Public dx(,) As Vector
    Public nrm(,) As Vector
    Public pow(,) As Double
    Public ik(,) As Vector
    Public im(,) As Vector
    Public r(,) As Double
    Public aa(,) As Double
    Public al(,) As Double
    Public ap(,) As Integer
    Public b() As Double
    Public c() As Double
    Public d() As Vector
    Public coef() As Double = {1.0, 2.0, 2.0, 1.0}

    Public swa As Boolean = True '距離に関係する力（万有引力，弾性力など）
    Public swb As Boolean = True '速さに比例する力（粘性抵抗）
    Public swc As Boolean = True '速さの二乗に比例する力（慣性抵抗）
    Public swd As Boolean = True '一定の力（地上の重力など）

    'DoubleCeil補助関数
    Public Function AndExponent(ByVal db() As Byte, ByVal ee() As Byte) As Byte()
        Dim ret(7) As Byte
        Dim i As Integer
        For i = 0 To 5
            ret(i) = db(i)
        Next
        ret(6) = (db(6) And (15))
        ret(6) = (ret(6)) Or (ee(0) << (4))
        ret(7) = db(7)
        Return ret
    End Function

    Public Function Exponent2Bytes(ByVal i As Int64) As Byte()
        Dim ret() As Byte = BitConverter.GetBytes(i)
        Return ret
    End Function

    Public Function Bytes2Exponent(ByVal b() As Byte) As Int64
        Dim ret(7) As Byte
        ret(0) = ((b(6) And (240)) >> (4))
        Return BitConverter.ToInt64(ret, 0)
    End Function

    Public Function AndFraction(ByVal db() As Byte, ByVal df() As Byte) As Byte()
        Dim ret(7) As Byte
        Dim i As Integer
        For i = 0 To 5
            ret(i) = df(i)
        Next
        ret(6) = (df(6) And (15))
        ret(6) = (ret(6)) Or (db(6) And (240))
        ret(7) = db(7)
        Return ret
    End Function

    Public Function Fraction2Bytes(ByVal i As Int64) As Byte()
        Dim ret() As Byte = BitConverter.GetBytes(i)
        Return ret
    End Function

    Public Function Bytes2Fraction(ByVal b() As Byte) As Int64
        Dim ret(7) As Byte
        Dim i As Integer
        For i = 0 To 5
            ret(i) = b(i)
        Next
        ret(6) = (b(6) And (15))
        ret(7) = 0
        Return BitConverter.ToInt64(ret, 0)
    End Function

    Public Function Bytes2Double(ByVal b() As Byte) As Double
        Return BitConverter.ToDouble(b, 0)
    End Function

    Public Function Double2Bytes(ByVal d As Double) As Byte()
        Dim ret() As Byte = BitConverter.GetBytes(d)
        Return ret
    End Function

    '1/2^(52-n)の桁上げ
    Public Function DoubleCeil(ByVal d As Double, ByVal n As Integer) As Double
        Dim db() As Byte = Double2Bytes(d)
        Dim di As Int64 = Bytes2Fraction(db)
        Dim o As Int64 = 1
        Dim p As Integer
        For p = 1 To n - 1
            o = o * 2
        Next
        di = di + o
        Dim df() As Byte = Fraction2Bytes(di)
        If (df(6) And (16)) Then
            Dim ei As Int64 = Bytes2Exponent(db)
            ei = ei + 1
            Dim ee() As Byte = Exponent2Bytes(ei)
            db = AndExponent(db, ee)
        End If
        db = AndFraction(db, df)
        Dim ret As Double = Bytes2Double(db)
        Return ret
    End Function

    '速度加速
    Public Function VelocityAccl2D(ByVal v As Vector, ByVal a As Vector, ByVal dt As Double) As Vector
        If a.Abs() = 0.0 Then
            Return v
        End If
        If 1.0 < v.Abs() Then
            If v.x(1) <= v.x(0) Then
                v.x(1) = Math.Sqrt(1.0 - v.x(0) * v.x(0))
            End If
            If v.x(0) < v.x(1) Then
                v.x(0) = Math.Sqrt(1.0 - v.x(1) * v.x(1))
            End If
        End If

        Dim va As Double = v.Abs()
        Dim aa As Double = a.Abs()

        Dim vab As Double = Math.Tanh(aa * dt)
        Dim vac As Double = ((va + vab) / (va * vab + 1.0))

        Dim ret As Vector = v + a * dt

        If 1.0 < vac Then
            ret = ret / vac
        End If

        Return ret
    End Function

    '速度加速
    Public Function VelocityAccl3D(ByVal v As Vector, ByVal a As Vector, ByVal dt As Double) As Vector
        If a.Abs() = 0.0 Then
            Return v
        End If
        If 1.0 < v.Abs() Then
            If v.x(1) <= v.x(0) And v.x(2) <= v.x(0) Then
                v.x(1) = Math.Sqrt((1.0 - v.x(0) * v.x(0)) / 2.0)
                v.x(2) = Math.Sqrt((1.0 - v.x(0) * v.x(0)) / 2.0)
            End If
            If v.x(2) <= v.x(1) And v.x(0) < v.x(1) Then
                v.x(2) = Math.Sqrt((1.0 - v.x(1) * v.x(1)) / 2.0)
                v.x(0) = Math.Sqrt((1.0 - v.x(1) * v.x(1)) / 2.0)
            End If
            If v.x(0) < v.x(2) And v.x(1) < v.x(2) Then
                v.x(0) = Math.Sqrt((1.0 - v.x(2) * v.x(2)) / 2.0)
                v.x(1) = Math.Sqrt((1.0 - v.x(2) * v.x(2)) / 2.0)
            End If
        End If

        Dim va As Double = v.Abs()
        Dim aa As Double = a.Abs()

        Dim vab As Double = Math.Tanh(aa * dt)
        Dim vac As Double = ((va + vab) / (va * vab + 1.0))

        Dim ret As Vector = v + a * dt

        If 1.0 < vac Then
            ret = ret / vac
        End If

        Return ret
    End Function

    '速度合成
    Public Function VelocityAdd2D(ByVal v0 As Vector, ByVal v1 As Vector) As Vector
        VelocityAdd2D = New Vector(v0.x.Length)
        Dim ra As Double = v1.Abs()
        ra = DoubleCeil(ra, 1)
        Dim i As Integer
        If v0.Abs() = 0.0 Then
            If 1.0 < ra Then
                For i = 0 To v0.x.Length - 1
                    VelocityAdd2D.x(i) = v1.x(i) / ra
                Next
            Else
                VelocityAdd2D = v1
            End If
            Return VelocityAdd2D
        End If
        ra = v0.Abs()
        ra = DoubleCeil(ra, 1)
        If v1.Abs() = 0.0 Then
            If 1.0 < ra Then
                For i = 0 To v0.x.Length - 1
                    VelocityAdd2D.x(i) = v0.x(i) / ra
                Next
            Else
                VelocityAdd2D = v0
            End If
            Return VelocityAdd2D
        End If


        Dim ret As Vector = New Vector(3)
        Dim v As Double = v0.Abs()
        Dim v11 As Vector = New Vector(3)
        Dim v00 As Vector = New Vector(3)
        For i = 0 To v0.x.Length - 1
            v11.x(i) = v1.x(i)
            v00.x(i) = v0.x(i)
        Next
        v11.x(i) = 0.0
        v00.x(i) = 0.0
        Dim ux As Vector = ret.UnitVec(0)
        Dim crs As Vector = v00.Cross(ux)

        Dim theta As Double = v00.Theta(ux)
        If Double.IsNaN(theta) Then
            theta = v11.Theta(ux)
            If Double.IsNaN(theta) Then
                theta = 0.0
            Else
                v00 = v11.RotAxis(crs, theta)
            End If
        Else
            v00 = v00.RotAxis(crs, theta)
        End If
        v11 = v11.RotAxis(crs, theta)
        If v11.x(0) * v = -1.0 Then
            v11.x(0) = -0.99999999999999989
        End If

        ret.x(0) = ((v11.x(0) + v) / (v11.x(0) * v + 1.0))
        ret.x(1) = ((v11.x(1)) / (v11.x(0) * v + 1.0)) * Math.Sqrt(1.0 - v * v)
        ret.x(2) = ((v11.x(2)) / (v11.x(0) * v + 1.0)) * Math.Sqrt(1.0 - v * v)
        ret = ret.RotAxis(crs, -theta)
        ra = ret.Abs()
        ra = DoubleCeil(ra, 1)
        If 1.0 < ra Then
            For i = 0 To v0.x.Length - 1
                VelocityAdd2D.x(i) = ret.x(i) / ra
            Next
        Else
            For i = 0 To v0.x.Length - 1
                VelocityAdd2D.x(i) = ret.x(i)
            Next
        End If
    End Function

    '速度合成
    Public Function VelocityAdd3D(ByVal v0 As Vector, ByVal v1 As Vector) As Vector
        VelocityAdd3D = New Vector(v0.x.Length)
        Dim ra As Double = v1.Abs()
        ra = DoubleCeil(ra, 1)
        Dim i As Integer
        If v0.Abs() = 0.0 Then
            If 1.0 < ra Then
                For i = 0 To v0.x.Length - 1
                    VelocityAdd3D.x(i) = v1.x(i) / ra
                Next
            Else
                VelocityAdd3D = v1
            End If
            Return VelocityAdd3D
        End If
        ra = v0.Abs()
        ra = DoubleCeil(ra, 1)
        If v1.Abs() = 0.0 Then
            If 1.0 < ra Then
                For i = 0 To v0.x.Length - 1
                    VelocityAdd3D.x(i) = v0.x(i) / ra
                Next
            Else
                VelocityAdd3D = v0
            End If
            Return VelocityAdd3D
        End If


        Dim ret As Vector = New Vector(3)
        Dim v As Double = v0.Abs()
        Dim v11 As Vector = New Vector(3)
        Dim v00 As Vector = New Vector(3)
        For i = 0 To v0.x.Length - 1
            v11.x(i) = v1.x(i)
            v00.x(i) = v0.x(i)
        Next
        Dim ux As Vector = ret.UnitVec(0)
        Dim crs As Vector = v00.Cross(ux)

        Dim theta As Double = v00.Theta(ux)
        If Double.IsNaN(theta) Then
            theta = v11.Theta(ux)
            If Double.IsNaN(theta) Then
                theta = 0.0
            Else
                v00 = v11.RotAxis(crs, theta)
            End If
        Else
            v00 = v00.RotAxis(crs, theta)
        End If
        v11 = v11.RotAxis(crs, theta)

        ret.x(0) = ((v11.x(0) + v) / (v11.x(0) * v + 1.0))
        ret.x(1) = ((v11.x(1)) / (v11.x(0) * v + 1.0)) * Math.Sqrt(1.0 - v * v)
        ret.x(2) = ((v11.x(2)) / (v11.x(0) * v + 1.0)) * Math.Sqrt(1.0 - v * v)
        ret = ret.RotAxis(crs, -theta)
        ra = ret.Abs()
        ra = DoubleCeil(ra, 1)
        If 1.0 < ra Then
            For i = 0 To v0.x.Length - 1
                VelocityAdd3D.x(i) = ret.x(i) / ra
            Next
        Else
            For i = 0 To v0.x.Length - 1
                VelocityAdd3D.x(i) = ret.x(i)
            Next
        End If
    End Function


    'シュワルツシルト半径
    Public Function GetSRadius(ByVal mass As Double, ByVal cc As Double, ByVal cg As Double) As Double
        Return ((2.0 * cg * mass) / (cc * cc))
    End Function

    'シュワルツシルト補正項
    Public Function ToSchwartz(ByVal v As Double) As Double
        ToSchwartz = 3.0 * v * v
    End Function

    '加速度を求める
    Public Function GetA(ByVal r As Double, ByVal m As Double, ByVal mr As Double, ByVal v As Double) As Double
        If r = 0.0 Then
            Return 0.0
        End If
        Dim a As Double = 1.0
        If mr <= r Then
            mr = r
        Else
            a = r / mr
        End If
        Dim g As Double = CNST_G * (m / mr / mr) * a
        g = g * (1.0 + ToSchwartz(v))
        If g < 0.0 Or Double.IsNaN(g) Then
            Return 0.0
        End If
        GetA = (g / CNST_C)
    End Function

    '//加速度の計算
    Public Sub accel()
        Dim i As Integer
        Dim j As Integer
        Dim k As Integer
        Dim fw As Double
        Dim fw1 As Double


        If swa Or swc Then
            For i = 0 To n - 1
                If swa Then
                    For j = i + 1 To n
                        fw = 0.0
                        dx(i, j) = mp(i).x1 - mp(j).x1
                        For k = 0 To dms
                            fw += dx(i, j).x(k) * dx(i, j).x(k)
                        Next
                        r(i, j) = Math.Sqrt(fw)

                        For k = 0 To dms
                            If Not r(i, j) = 0.0 Then
                                nrm(i, j).x(k) = dx(i, j).x(k) / r(i, j)
                            Else
                                nrm(i, j).x(0) = 1.0
                            End If
                        Next
                        r(i, j) = r(i, j) - al(i, j)

                        If ap(i, j) = 0 Then
                            pow(i, j) = 1.0
                        End If
                        If ap(i, j) = -2 Then
                            If Not r(i, j) = 0.0 Then
                                pow(i, j) = 1.0 / r(i, j) / r(i, j)
                            Else
                                pow(i, j) = 0.0
                            End If
                        End If

                        If ap(j, i) = 0 Then
                            pow(j, i) = 1.0
                        End If
                        If ap(j, i) = -2 Then
                            If Not r(i, j) = 0.0 Then
                                pow(j, i) = 1.0 / r(i, j) / r(i, j)
                            Else
                                pow(j, i) = 0.0
                            End If
                        End If

                        If ap(i, j) = -1 Then
                            Dim a1 As Double
                            '軌道成分速度
                            Dim ov As New Vector(3)
                            Dim dx2 As New Vector(3)
                            Dim dx3 As New Vector(3)
                            Dim dx1 As New Vector(3)
                            Dim vv1 As New Vector(3)
                            For k = 0 To dx(i, j).x.Length - 1
                                dx1.x(k) = dx(i, j).x(k)
                                vv1.x(k) = mp(i).v1.x(k)
                            Next
                            If Not dx1.Abs = 0.0 Then
                                If dx1 = vv1 Then
                                    If dx1 = dx2.UnitVec(0) Then
                                        dx2 = dx1.Cross(dx2.UnitVec(1))
                                    Else
                                        dx2 = dx1.Cross(dx2.UnitVec(0))
                                    End If
                                Else
                                    dx2 = dx1.Cross(vv1)
                                End If
                                If dx1 = dx2 Then
                                    'ERROR
                                    dx3 = dx1
                                Else
                                    dx3 = dx1.Cross(dx2)
                                End If
                                ov = vv1.ProjectAxis(dx3)
                            End If
                            a1 = GetA(r(i, j), mp(j).mass, mp(j).r, ov.Abs())
                            pow(i, j) = a1
                        End If
                        If ap(j, i) = -1 Then
                            Dim a1 As Double
                            '軌道成分速度
                            Dim ov As New Vector(3)
                            Dim dx2 As New Vector(3)
                            Dim dx3 As New Vector(3)
                            Dim dx1 As New Vector(3)
                            Dim vv1 As New Vector(3)
                            For k = 0 To dx(i, j).x.Length - 1
                                dx1.x(k) = -dx(i, j).x(k)
                                vv1.x(k) = mp(j).v1.x(k)
                            Next
                            If Not dx1.Abs = 0.0 Then
                                If dx1 = vv1 Then
                                    If dx1 = dx2.UnitVec(0) Then
                                        dx2 = dx1.Cross(dx2.UnitVec(1))
                                    Else
                                        dx2 = dx1.Cross(dx2.UnitVec(0))
                                    End If
                                Else
                                    dx2 = dx1.Cross(vv1)
                                End If
                                If dx1 = dx2 Then
                                    'ERROR
                                    dx3 = dx1
                                Else
                                    dx3 = dx1.Cross(dx2)
                                End If
                                ov = vv1.ProjectAxis(dx3)
                            End If
                            a1 = GetA(r(i, j), mp(i).mass, mp(i).r, ov.Abs())
                            pow(j, i) = a1
                        End If

                        If ap(i, j) = 1 Then
                            pow(i, j) = r(i, j)
                        End If
                        If ap(j, i) = 1 Then
                            pow(j, i) = r(i, j)
                        End If

                        pow(i, j) = pow(i, j) * aa(i, j)
                        pow(j, i) = pow(j, i) * aa(j, i)
                    Next
                End If

                fw1 = 0.0
                For k = 0 To dms
                    fw1 += mp(i).v1.x(k) * mp(i).v1.x(k)
                Next
                mp(i).va = Math.Sqrt(fw1)
                For k = 0 To dms
                    If Not mp(i).va = 0.0 Then
                        mp(i).vn.x(k) = mp(i).v1.x(k) / mp(i).va
                    End If
                Next
            Next
        End If

        For i = 0 To n
            For k = 0 To dms
                mp(i).a.x(k) = 0.0 '//clear
            Next
        Next


        For i = 0 To n
            If swa Then '//２物体間の相互作用
                If Not i = n Then
                    For j = i + 1 To n
                        Dim a As Vector = New Vector(dms + 1)
                        For k = 0 To dms
                            a.x(k) = ((nrm(i, j).x(k) * pow(i, j) / (mp(i).mass)))
                            mp(i).a.x(k) = mp(i).a.x(k) + a.x(k)

                            a.x(k) = ((-nrm(i, j).x(k) * pow(j, i) / (mp(j).mass)))
                            mp(j).a.x(k) = mp(j).a.x(k) + a.x(k)
                        Next
                    Next
                End If
            End If
            If swb Then '//速さに比例する力（粘性抵抗）
                Dim a As Vector = New Vector(dms + 1)
                For k = 0 To dms
                    a.x(k) = ((mp(i).v1.x(k) * b(i)) / (mp(i).mass))
                    mp(i).a.x(k) = mp(i).a.x(k) + a.x(k)
                Next
            End If
            If swc Then '//速さの二乗に比例する抵抗（慣性抵抗）
                Dim a As Vector = New Vector(dms + 1)
                For k = 0 To dms
                    a.x(k) = ((mp(i).vn.x(k) * mp(i).va * mp(i).va * c(i)) / (mp(i).mass))
                    mp(i).a.x(k) = mp(i).a.x(k) + a.x(k)
                Next
            End If
            If swd Then '//一定の力（重力など）
                Dim a As Vector = New Vector(dms + 1)
                For k = 0 To dms
                    a.x(k) = ((d(i).x(k)) / (mp(i).mass))
                    mp(i).a.x(k) = mp(i).a.x(k) + a.x(k)
                Next
            End If
        Next

    End Sub

    'ルンゲ-クッタ法
    Public Sub UpdateFrame()
        Dim i As Integer
        Dim k As Integer
        Dim l As Integer


        '設定
        For i = 0 To n
            For k = 0 To dms
                mp(i).x1.x(k) = mp(i).x.x(k)
                mp(i).x0.x(k) = mp(i).x.x(k)
                mp(i).v1.x(k) = mp(i).v.x(k)
                mp(i).w.x(k) = 0.0
                mp(i).w1.x(k) = 0.0
            Next
        Next

        'ルンゲ-クッタ法
        accel()
        For l = 0 To 2
            For i = 0 To n
                Dim v01 As Vector = New Vector(dms + 1)
                Dim v02 As Vector = New Vector(dms + 1)
                Dim v1 As Vector = New Vector(dms + 1)
                Dim v2 As Vector = New Vector(dms + 1)
                ik(l, i) = mp(i).v1 * dt * CNST_C
                mp(i).x1 = mp(i).x + ik(l, i) / coef(l + 1)
                mp(i).x0 = mp(i).x1
                mp(i).w = mp(i).w + ik(l, i) * coef(l)


                Dim av As Vector = New Vector(0)
                If dms = 2 Then
                    av = VelocityAccl3D(New Vector(3), mp(i).a, dt)
                ElseIf dms = 1 Then
                    av = VelocityAccl2D(New Vector(2), mp(i).a, dt)
                End If
                For k = 0 To dms
                    im(l, i).x(k) = av.x(k)
                Next

                v1 = im(l, i) / coef(l + 1)
                v2 = im(l, i) * coef(l)
                For k = 0 To dms
                    mp(i).v1.x(k) = (mp(i).v.x(k) + v1.x(k))
                    mp(i).w1.x(k) = (mp(i).w1.x(k) + v2.x(k))
                Next

            Next
            accel()
        Next
        For i = 0 To n
            Dim v01 As Vector = New Vector(dms + 1)
            Dim v02 As Vector = New Vector(dms + 1)
            Dim v1 As Vector = New Vector(dms + 1)
            Dim v2 As Vector = New Vector(dms + 1)
            ik(l, i) = mp(i).v1 * dt * CNST_C
            mp(i).x1 = mp(i).x + ((mp(i).w + ik(3, i)) / 6.0)

            Dim av As Vector = New Vector(0)
            If dms = 2 Then
                av = VelocityAccl3D(New Vector(3), mp(i).a, dt)
            ElseIf dms = 1 Then
                av = VelocityAccl2D(New Vector(2), mp(i).a, dt)
            End If
            For k = 0 To dms
                im(l, i).x(k) = av.x(k)
            Next

            v1 = im(3, i)
            For k = 0 To dms
                v01.x(k) = mp(i).w1.x(k)
                v02.x(k) = mp(i).v.x(k)
            Next
            If dms = 2 Then
                For k = 0 To dms
                    v2.x(k) = (v01.x(k) + v1.x(k)) / 6.0
                Next
                mp(i).v1 = VelocityAdd3D(v02, v2)
            ElseIf dms = 1 Then
                For k = 0 To dms
                    v2.x(k) = (v01.x(k) + v1.x(k)) / 6.0
                Next
                mp(i).v1 = VelocityAdd2D(v02, v2)
            End If
        Next


        'パラメータ適用
        For i = 0 To n
            If 1.0 < mp(i).v1.Abs() Then
                i = i
            End If
            For k = 0 To dms
                mp(i).x.x(k) = mp(i).x1.x(k)
                mp(i).v.x(k) = mp(i).v1.x(k)
            Next
        Next

    End Sub

    'ルンゲ-クッタ法初期設定
    Public Sub Init(ByVal pmp() As MassPoint, ByVal pdt As Double, ByVal cc As Double, ByVal cg As Double)
        Me.mp = pmp
        Me.dt = pdt

        If Not cc = 0.0 Then
            Me.CNST_C = cc
        End If
        If Not cg = 0.0 Then
            Me.CNST_G = cg
        End If

        dms = pmp(0).x.x.Length - 1
        n = pmp.Length - 1


        ReDim dx(n, n)
        ReDim nrm(n, n)
        ReDim pow(n, n)
        ReDim ik(3, n)
        ReDim im(3, n)
        ReDim r(n, n)
        ReDim aa(n, n)
        ReDim al(n, n)
        ReDim ap(n, n)
        ReDim b(n)
        ReDim c(n)
        ReDim d(n)


        Dim i As Integer
        Dim j As Integer
        Dim k As Integer = pmp(0).x.x.Length

        For i = 0 To n
            For j = i To n
                dx(i, j) = New Vector(k)
                nrm(i, j) = New Vector(k)
                pow(i, j) = 0.0
                r(i, j) = 0.0

                aa(i, j) = -(mp(i).mass)
                al(i, j) = 0.0
                ap(i, j) = -1   '相対性理論万有引力


                dx(j, i) = New Vector(k)
                nrm(j, i) = New Vector(k)
                pow(j, i) = 0.0
                r(j, i) = 0.0

                aa(j, i) = -(mp(j).mass)
                al(j, i) = 0.0
                ap(j, i) = -1   '相対性理論万有引力
            Next
        Next
        For i = 0 To 3
            For j = 0 To n
                ik(i, j) = New Vector(k)
                im(i, j) = New Vector(k)
            Next
        Next
        For i = 0 To n
            b(i) = 0.0
            c(i) = 0.0
            d(i) = New Vector(k)
        Next
        '//---力の設定-------	
        swa = True '//２物体間の相互作用
        '//強さ
        'aa(0, 0) = -50000
        '//aa[1][0]=-50000; aa[1][1]=0.;
        '//aa[2][0]=100000.; aa[2][1]=0.; aa[2][2]=0.;
        '//力のべき（-2は万有引力やクーロン力，1はバネの弾性力）
        'ap(0, 0) = -2
        '//ap[1][0]=-2; ap[1][1]=0;
        '//ap[2][0]=100000.; ap[2][1]=0.; ap[2][2]=0.;
        '//バネの長さ
        'al(0, 0) = 0.0
        '//al[1][0]=0.; al[1][1]=0.;
        '//al[2][0]=100000.; al[2][1]=0.; al[2][2]=0.;


        'If n > 0 Then '//対称化（作用・反作用の法則）
        'For i = 1 To n
        'For j = 0 To i - 1
        'aa(j, i) = aa(i, j)
        'ap(j, i) = ap(i, j)
        'al(j, i) = al(i, j)
        'Next
        'Next
        'End If

        swb = False '//速さに比例する力（粘性抵抗）
        '//b[0]=0.; b[1]=0.; b[2]=0.;

        swc = False '//速さの二乗に比例する抵抗（慣性抵抗）
        'swc = True '//速さの二乗に比例する抵抗（慣性抵抗）
        'Dim P As Double = 0.0000000000028
        'Dim PACD As Double = 800000000000.0
        'Dim AR As Double = -(1.0 / 2.0) * P * PACD

        'c(0) = 0.0
        'c(1) = AR
        'c(2) = 0.0

        swd = False '//一定の力（重力など）
        '//d[0][0]=0.; d[0][1]=9.8*mass[0];
        '//d[1][0]=0.; d[1][1]=9.8*mass[1];
        '//d[2][0]=0.; d[2][1]=9.8*mass[2];		

    End Sub

    'ルンゲ-クッタ法初期設定
    Public Sub Init(ByVal pmp() As MassPoint, ByVal pdt As Double)
        Init(pmp, pdt, 0.0, 0.0)
    End Sub


End Class
