﻿'Programed by NAS6

Public Class SphereData
    Implements ICloneable

    Public v As Vector                  '中心座標
    Public r As Double                  '半径
    Public vcol As System.Drawing.Color '頂点カラー
    Public tcol As System.Drawing.Color '面カラー
    Public mtr As Material              '物性

    Public Sub New()
        v = New Vector(4)
        r = 0.0
        vcol = Color.White
        tcol = Color.Black
        mtr = New Material
    End Sub

    Public Sub New(ByVal pv As Vector, ByVal pr As Double, ByVal pvc As System.Drawing.Color, ByVal ptc As System.Drawing.Color, ByVal pm As Material)
        v = pv.Clone()
        r = pr
        vcol = pvc
        tcol = ptc
        mtr = pm.Clone()
    End Sub

    Public Function Clone() As Object Implements ICloneable.Clone
        Dim ret As New SphereData
        ret.v = Me.v.Clone()
        ret.r = Me.r
        ret.vcol = Me.vcol
        ret.tcol = Me.tcol
        ret.mtr = Me.mtr.Clone()
        Return ret
    End Function
End Class

Public Class Sphere
    Implements IComparable
    Implements ICloneable

    Public Local As New Local       'ローカル行列群
    Public v As SphereData          '球  
    Public Range As Double = 1.0    '範囲
    Public Base As Double = 2.0     '基数
    Public Label As New Label       'ラベル

    Public BInf As Boolean = True
    Public Frs As Frustum


    Public Function Clone() As Object Implements ICloneable.Clone
        Dim ret As New Sphere(Me.Local.Clone(), Me.v.Clone(), Me.Range, Me.Base, Me.Label.Clone())
        Return ret
    End Function

    Public Function CompareTo(obj As Object) As Integer Implements IComparable.CompareTo
        CompareTo = 0
        Dim lh As Sphere = Me
        Dim rh As Sphere = obj
        If Not (lh.Base = rh.Base) Then
            CompareTo = CompareTo Or 1
        End If
    End Function

    Public Overrides Function Equals(obj As Object) As Boolean
        Dim ret As Integer = CompareTo(obj)
        If ret = 0 Then
            Return True
        Else
            Return False
        End If
    End Function

    Public Shared Operator =(ByVal lh As Sphere, ByVal rh As Sphere) As Boolean
        Dim ret As Integer = lh.CompareTo(rh)
        If ret = 0 Then
            Return True
        Else
            Return False
        End If
    End Operator

    Public Shared Operator <>(ByVal lh As Sphere, ByVal rh As Sphere) As Boolean
        Return (Not lh = rh)
    End Operator




    Public Sub New(ByVal lcl As Local, ByVal pv As SphereData, ByVal fr As Frustum)
        Local = lcl.Clone()
        v = pv.Clone()
        BInf = False
        Frs = fr.Clone()
    End Sub

    Public Sub New(ByVal lcl As Local, ByVal pv As SphereData, ByVal fr As Frustum, ByVal lbl As Label)
        Local = lcl.Clone()
        Label = lbl.Clone()
        v = pv.Clone()
        BInf = False
        Frs = fr.Clone()
    End Sub


    Public Sub New(ByVal lcl As Local, ByVal pv As SphereData)
        Local = lcl.Clone()
        v = pv.Clone()
    End Sub

    Public Sub New(ByVal lcl As Local, ByVal pv As SphereData, ByVal rg As Double, ByVal bs As Double)
        Local = lcl.Clone()
        v = pv.Clone()
        Range = rg
        Base = bs
    End Sub


    Public Sub New(ByVal lcl As Local, ByVal pv As SphereData, ByVal lbl As Label)
        Local = lcl.Clone()
        Label = lbl.Clone()
        v = pv.Clone()
    End Sub

    Public Sub New(ByVal lcl As Local, ByVal pv As SphereData, ByVal rg As Double, ByVal bs As Double, ByVal lbl As Label)
        Local = lcl.Clone()
        Label = lbl.Clone()
        v = pv.Clone()
        Range = rg
        Base = bs
    End Sub

    '描画
    'cmr:カメラ行列群,gPic:グラフィックスオブジェクト,w:画面幅,h:画面高さ,l:左オフセット,t:上オフセット,b:true:通常描画,false:ワイヤーフレーム陰面消去なし
    Public Sub Render(ByVal cmr As Local, ByVal gPic As Graphics, ByVal w As Integer, ByVal h As Integer, ByVal l As Integer, ByVal t As Integer, ByVal b As Boolean)
        Dim InfWK As New Infinity(Range, Base)
        Dim dt As Double
        Dim MatWK As New Matrix(4)
        If Local.bcmr Then
            MatWK = MatWK.UnitMat()
        Else
            MatWK = cmr.Mat()
            MatWK = MatWK.InverseMat(dt)
        End If
        MatWK *= Local.Mat()
        MatWK = MatWK.Homogeneous()


        Dim VecWK1 As Vector
        Dim VecWK2 As Vector
        Dim VecWK1F As Vector
        Dim VecWK2F As Vector
        VecWK1 = v.v.Clone()
        VecWK1 = MatWK * VecWK1
        VecWK1.x(1) -= v.r
        VecWK1.x(2) -= v.r
        VecWK1 = VecWK1.Homogeneous()
        VecWK2 = v.v.Clone()
        VecWK2 = MatWK * VecWK2
        VecWK2.x(1) += v.r
        VecWK2.x(2) += v.r
        VecWK2 = VecWK2.Homogeneous()

        If BInf Then
            VecWK1F = InfWK.FrustumInf(VecWK1)
            VecWK2F = InfWK.FrustumInf(VecWK2)
        Else
            VecWK1F = Frs.Frustum(VecWK1)
            VecWK2F = Frs.Frustum(VecWK2)
            VecWK1F.x(0) = 1.0
            VecWK1F.x(1) = -VecWK1F.x(1) / VecWK1F.x(3)
            VecWK1F.x(2) = -VecWK1F.x(2) / VecWK1F.x(3)
            VecWK1F.x(3) = VecWK1.x(3)
            VecWK2F.x(0) = 1.0
            VecWK2F.x(1) = -VecWK2F.x(1) / VecWK2F.x(3)
            VecWK2F.x(2) = -VecWK2F.x(2) / VecWK2F.x(3)
            VecWK2F.x(3) = VecWK2.x(3)
        End If

        If v.r <= VecWK1.x(3) Then
            Dim pen As New System.Drawing.Pen(v.vcol)

            Dim xx As Integer = CType(VecWK1F.x(1) * CType(w / 2, Double), Integer) + l
            Dim yy As Integer = CType(VecWK1F.x(2) * CType(h / 2, Double), Integer) + t
            Dim ww As Integer = CType(Math.Abs(CType(VecWK2F.x(1) * CType(w / 2, Double), Integer) + l - xx), Integer)
            Dim hh As Integer = CType(Math.Abs(CType(VecWK2F.x(2) * CType(h / 2, Double), Integer) + l - yy), Integer)

            Dim rect As New Rectangle(xx, yy, ww, hh)

            If Not v.tcol = Color.Black And b Then
                Dim brs As New System.Drawing.SolidBrush(v.tcol)
                gPic.FillEllipse(brs, rect)
            End If
            gPic.DrawEllipse(pen, rect)
        End If
    End Sub

End Class
