﻿'ベクトル
Public Class Vector
    Public x() As Double

    Public Function Clone() As Vector
        Dim d() As Double = Me.x.Clone()
        Return New Vector(d)
    End Function

    Public Sub New(ByVal n As Integer)
        ReDim x(n - 1)
        Dim i As Integer
        For i = 0 To n - 1
            x(i) = 0.0
        Next
    End Sub
    Public Sub New(ByVal px() As Double)
        x = px
    End Sub

    '四則演算(便宜上も含む)
    Public Shared Operator +(ByVal lh As Vector, ByVal rh As Vector) As Vector
        Dim ret As Vector = New Vector(lh.x.Length)
        If Not lh.x.Length = rh.x.Length Then
            Return New Vector(lh.x.Length)
        End If
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            ret.x(i) = lh.x(i) + rh.x(i)
        Next
        Return ret
    End Operator

    Public Shared Operator -(ByVal lh As Vector, ByVal rh As Vector) As Vector
        Dim ret As Vector = New Vector(lh.x.Length)
        If Not lh.x.Length = rh.x.Length Then
            Return New Vector(lh.x.Length)
        End If
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            ret.x(i) = lh.x(i) - rh.x(i)
        Next
        Return ret
    End Operator

    Public Shared Operator *(ByVal lh As Vector, ByVal rh As Vector) As Double
        Dim ret As Double = 0.0
        If Not lh.x.Length = rh.x.Length Then
            Return ret
        End If
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            ret = (ret + (lh.x(i) * rh.x(i)))
        Next
        Return ret
    End Operator

    Public Shared Operator /(ByVal lh As Vector, ByVal rh As Vector) As Double
        Dim ret As Double = 0.0
        If Not lh.x.Length = rh.x.Length Then
            Return ret
        End If
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            If rh.x(i) = 0.0 Then
                Return 0.0
            End If
            ret = (ret + (lh.x(i) / rh.x(i)))
        Next
        Return ret
    End Operator

    Public Shared Operator +(ByVal lh As Vector, ByVal rh As Double) As Vector
        Dim ret As Vector = New Vector(lh.x.Length)
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            ret.x(i) = lh.x(i) + rh
        Next
        Return ret
    End Operator

    Public Shared Operator -(ByVal lh As Vector, ByVal rh As Double) As Vector
        Dim ret As Vector = New Vector(lh.x.Length)
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            ret.x(i) = lh.x(i) - rh
        Next
        Return ret
    End Operator

    Public Shared Operator *(ByVal lh As Vector, ByVal rh As Double) As Vector
        Dim ret As Vector = New Vector(lh.x.Length)
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            ret.x(i) = lh.x(i) * rh
        Next
        Return ret
    End Operator

    Public Shared Operator /(ByVal lh As Vector, ByVal rh As Double) As Vector
        Dim ret As Vector = New Vector(lh.x.Length)
        If rh = 0.0 Then
            Return New Vector(lh.x.Length)
        End If
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            ret.x(i) = lh.x(i) / rh
        Next
        Return ret
    End Operator

    Public Shared Operator =(ByVal lh As Vector, ByVal rh As Vector) As Boolean
        If Not lh.x.Length = rh.x.Length Then
            Return False
        End If
        Dim i As Integer
        For i = 0 To lh.x.Length - 1
            If Not lh.x(i) = rh.x(i) Then
                Return False
            End If
        Next
        Return True
    End Operator

    Public Shared Operator <>(ByVal lh As Vector, ByVal rh As Vector) As Boolean
        Return (Not lh = rh)
    End Operator


    '内積
    Public Function Dot(ByVal rh As Vector) As Double
        Dim sum As Double = 0.0
        Dim i As Integer
        For i = 0 To Me.x.Length - 1
            sum = sum + Me.x(i) * rh.x(i)
        Next
        Dot = sum
    End Function

    '外積
    Public Function Cross2D(ByVal rh As Vector) As Double
        If Not Me.x.Length = 2 Or Not Me.x.Length = rh.x.Length Then
            Return 0.0
        End If
        Return Me.x(0) * rh.x(1) - Me.x(1) * rh.x(0)
    End Function

    '外積
    Public Function Cross3D(ByVal rh As Vector) As Vector
        If Not Me.x.Length = 3 Or Not Me.x.Length = rh.x.Length Then
            Return New Vector(Me.x.Length)
        End If
        Dim d(2) As Double
        d(0) = Me.x(1) * rh.x(2) - Me.x(2) * rh.x(1)
        d(1) = Me.x(2) * rh.x(0) - Me.x(0) * rh.x(2)
        d(2) = Me.x(0) * rh.x(1) - Me.x(1) * rh.x(0)
        Return New Vector(d)
    End Function

    '原点ベクトル
    Public Function ZeroVec() As Vector
        Dim i As Integer
        Dim d0(Me.x.Length - 1) As Double
        For i = 0 To Me.x.Length - 1
            d0(i) = 0.0
        Next
        ZeroVec = New Vector(d0)
    End Function

    '単位ベクトル
    Public Function UnitVec(ByVal a As Integer) As Vector
        If Me.x.Length - 1 < a Then
            Return ZeroVec()
        End If
        UnitVec = ZeroVec()
        UnitVec.x(a) = 1.0
    End Function

    'ノーマライズ
    Public Function NormalVec(ByVal a As Vector) As Vector
        Return a / a.Abs()
    End Function

    'ノーマライズ
    Public Function NormalVec(ByVal a As Vector, ByVal b As Vector) As Vector
        Dim ab As Vector = b - a
        Return NormalVec(ab)
    End Function

    '絶対値
    Public Function Abs() As Double
        Dim sum As Double = 0.0
        Dim i As Integer
        For i = 0 To Me.x.Length - 1
            sum = sum + Me.x(i) * Me.x(i)
        Next
        Abs = Math.Sqrt(sum)
    End Function

    '方向余弦
    Public Function DirectionCosine() As Vector
        DirectionCosine = New Vector(Me.x.Length)
        Dim r As Double = Me.Abs()
        Dim i As Integer
        For i = 0 To Me.x.Length - 1
            DirectionCosine.x(i) = Me.x(i) / r
        Next
    End Function

    'なす角
    Public Function Theta(ByVal rh As Vector) As Double
        Dim s As Double = 1.0
        Dim dc As Double = 0.0
        If Me.x.Length = 2 Then
            Dim crs As Double = Me.Cross2D(rh)
            If crs < 0.0 Then
                s = -1.0
            End If
        ElseIf Me.x.Length = 3 Then
            'Dim crs As Vector = Me.Cross3D(rh)
            'If (crs.x(2) < 0.0) Then
            's = -1.0
            'End If
        End If
        dc = Me.Dot(rh) / (Me.Abs() * rh.Abs())
        Return Math.Acos(dc) * s
    End Function

    'なす角
    Public Function ThetaN(ByVal rh As Vector) As Boolean
        Dim th As Double = Me.Theta(rh)
        If th <= -Math.PI / 2.0 Or Math.PI / 2.0 < th Then
            Return False
        End If
        Return True
    End Function

    '回転
    Public Function Rot(ByVal theta As Double) As Vector
        If Not Me.x.Length = 2 Then
            Return New Vector(Me.x.Length)
        End If
        Dim d0(1) As Double
        d0(0) = Me.x(0) * Math.Cos(theta) - Me.x(1) * Math.Sin(theta)
        d0(1) = Me.x(0) * Math.Sin(theta) + Me.x(1) * Math.Cos(theta)
        Return New Vector(d0)
    End Function

    '軸に対する回転
    Public Function RotAxis(ByVal axis As Vector, ByVal theta As Double) As Vector
        If Not Me.x.Length = 3 Or Not Me.x.Length = axis.x.Length Then
            Return New Vector(Me.x.Length)
        End If
        axis = axis / axis.Abs()
        Dim d0(2) As Double
        d0(0) = Me.x(0)
        d0(1) = Me.x(1)
        d0(2) = Me.x(2)
        Dim v0 As Vector = (New Vector(d0)) * Math.Cos(theta)
        Dim d1(2) As Double
        d1(0) = 0.0 - axis.x(2) * Me.x(1) + axis.x(1) * Me.x(2)
        d1(1) = axis.x(2) * Me.x(0) + 0.0 - axis.x(0) * Me.x(2)
        d1(2) = -axis.x(1) * Me.x(0) + axis.x(0) * Me.x(1) + 0.0
        Dim v1 As Vector = (New Vector(d1)) * Math.Sin(theta)
        Dim d2(2) As Double
        d2(0) = axis.x(0) * axis.x(0) * Me.x(0) + axis.x(0) * axis.x(1) * Me.x(1) + axis.x(0) * axis.x(2) * Me.x(2)
        d2(1) = axis.x(1) * axis.x(0) * Me.x(0) + axis.x(1) * axis.x(1) * Me.x(1) + axis.x(1) * axis.x(2) * Me.x(2)
        d2(2) = axis.x(2) * axis.x(0) * Me.x(0) + axis.x(2) * axis.x(1) * Me.x(1) + axis.x(2) * axis.x(2) * Me.x(2)
        Dim v2 As Vector = (New Vector(d2)) * (1.0 - Math.Cos(theta))
        Return (v0 + v1 + v2)
    End Function

    '軸に対する射影
    Public Function ProjectAxis(ByVal axis As Vector) As Vector
        If Not Me.x.Length = axis.x.Length Then
            Return New Vector(Me.x.Length)
        End If
        Return ((axis * (Me.Dot(axis))) / (axis.Abs * axis.Abs))
    End Function

    '点と直線の距離
    Public Function DistanceDotLine(ByVal p As Vector, ByVal a As Vector, ByVal b As Vector) As Double
        Dim ab As Vector = b - a
        Dim ap As Vector = p - a
        Dim aab As Double = ab.Abs()
        Dim pab As Double = (ab.ProjectAxis(ap)).Abs()
        If aab < pab Then
            Return 0.0
        End If
        Return Math.Sqrt(aab * aab - pab * pab)
    End Function

    '直線と直線の距離と最接近位置
    Public Function DistancePointLineLine(ByRef reta As Vector, ByRef retb As Vector, ByVal a0 As Vector, ByVal a1 As Vector, ByVal b0 As Vector, ByVal b1 As Vector) As Double
        Dim f As Double = 0.0
        Dim s As Double
        Dim t As Double
        Dim ab As Vector = b0 - a0
        Dim ba As Vector = a0 - b0
        Dim nva As Vector = NormalVec(a0, a1)
        Dim nvb As Vector
        If b0 = b1 Then
            nvb = nva * -1.0
        Else
            nvb = NormalVec(b0, b1)
        End If
        If a0 = a1 Then
            If b0 = b1 Then
                reta = a0
                retb = b0
                Return ab.Abs()
            Else
                nva = nvb * -1.0
            End If
        End If
        If (nva.Dot(nvb)) * (nva.Dot(nvb)) = 1.0 Then
            reta = a0
            retb = b0
            Return DistanceDotLine(b0, a0, a1)
        Else
            s = (1.0 / (1.0 - (nva.Dot(nvb)) * (nva.Dot(nvb)))) * (nva.Dot(ab) + nva.Dot(nvb) * nvb.Dot(ba))
            t = (1.0 / (1.0 - (nva.Dot(nvb)) * (nva.Dot(nvb)))) * (nvb.Dot(ba) + nva.Dot(nvb) * nva.Dot(ab))
            f = a0.Abs() * a0.Abs() + s * s + 2.0 * s * nva.Dot(a0) + b0.Abs() * b0.Abs() + t * t + 2.0 * t * nvb.Dot(b0) - 2.0 * a0.Dot(b0) - 2.0 * s * nva.Dot(b0) - 2.0 * t * nvb.Dot(a0) - 2.0 * s * t * nva.Dot(nvb)
        End If
        reta = a0 + nva * s
        retb = b0 + nvb * t
        Return Math.Sqrt(f)
    End Function

    '直線と直線の最接近位置
    Public Function PointLineLine(ByRef reta As Vector, ByRef retb As Vector, ByVal a0 As Vector, ByVal a1 As Vector, ByVal b0 As Vector, ByVal b1 As Vector) As Boolean
        Dim d As Double = DistancePointLineLine(reta, retb, a0, a1, b0, b1)
        If Not d = 0.0 Then
            Return False
        End If
        Return True
    End Function

    '直線と直線の距離
    Public Function DistanceLineLine(ByVal a0 As Vector, ByVal a1 As Vector, ByVal b0 As Vector, ByVal b1 As Vector) As Double
        Dim reta As Vector = New Vector(a0.x.Length)
        Dim retb As Vector = New Vector(a0.x.Length)
        Return DistancePointLineLine(reta, retb, a0, a1, b0, b1)
    End Function

End Class

